'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.callIn = callIn;

var _proxyequal = require('proxyequal');

var _cache = require('./cache');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var nothing = 'PROXY_EQUAL_NOTHING';
var emptyArray = [];

function addAffected(callGeneration, affected, object) {
  var key = (0, _proxyequal.getProxyKey)(object);
  if (key.fingerPrint.callGeneration === callGeneration) {
    affected[key.fingerPrint.index].resultAffected.push(key.suffix);
    return true;
  }
  return false;
}

var cycleMap = null;
var userShouldDive = null;
var returnTrue = function returnTrue() {
  return true;
};

var shouldDive = function shouldDive(line, key, object) {
  return (typeof line === 'undefined' ? 'undefined' : _typeof(line)) === 'object' && ((0, _proxyequal.isProxyfied)(line) || userShouldDive(line, key, object));
};

function deproxifyResult(callGeneration, result, affected, returnPureValue) {
  var deepDive = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

  var isInProxy = (0, _proxyequal.isProxyfied)(result);
  if (isInProxy) {
    if (addAffected(callGeneration, affected, result)) {
      var preResult = (0, _proxyequal.deproxify)(result);
      //return preResult;
      return deepDive ? preResult : deproxifyResult(callGeneration, preResult, affected, true, true);
    }
  }

  if ((typeof result === 'undefined' ? 'undefined' : _typeof(result)) === 'object') {
    var sub = Array.isArray(result) ? [] : {};
    var altered = false;

    if (result && Object.getOwnPropertyDescriptor(result, '__proxyequal_scanEnd')) {
      Object.defineProperty(result, '__proxyequal_scanEnd', {
        value: 'here was spread guard',
        configurable: true,
        enumerable: false
      });
    }

    Object.keys(result).forEach(function (i) {
      var data = result[i];
      var newResult = data;
      if (data && shouldDive(data, i, result)) {
        if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) === 'object') {
          if (!cycleMap.has(data)) {
            cycleMap.set(data, nothing);
            newResult = deproxifyResult(callGeneration, data, affected, false, deepDive);
            cycleMap.set(data, newResult);
          } else {
            newResult = cycleMap.get(data);
          }
        } else {
          newResult = deproxifyResult(callGeneration, data, affected, false, deepDive);
        }
      }
      if (data && newResult !== nothing && newResult !== data) {
        altered = true;
        sub[i] = newResult;
      } else {
        sub[i] = data;
      }
    });

    if (altered) {
      return sub;
    }
    return returnPureValue ? result : nothing;
  }

  return returnPureValue ? result : nothing;
}

function callIn(that, cache, args, func, memoizationDepth) {
  var proxyMap = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [];
  var options = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {};

  var proxies = args.map(function (arg, index) {
    if (arg && (typeof arg === 'undefined' ? 'undefined' : _typeof(arg)) === 'object') {
      var map = proxyMap[index];
      if (!map) {
        return proxyMap[index] = (0, _proxyequal.proxyState)(arg, { callGeneration: proxyMap, index: index });
      }
      map.reset();
      return map.replaceState(arg);
    }
    return undefined;
  });
  var callArgs = args.map(function (arg, index) {
    return proxies[index] ? proxies[index].state : arg;
  });

  //call the real function
  var preResult = func.call.apply(func, [that].concat(_toConsumableArray(callArgs)));
  proxies.forEach(function (proxy) {
    return proxy && proxy.seal();
  });

  var spreadDetected = [];
  var affected = proxies.map(function (proxy) {
    if (proxy) {
      if (proxy.spreadDetected !== false) {
        spreadDetected.push(proxy.spreadDetected);
      }
      var _affected = proxy.affected || emptyArray;

      return {
        useAffected: [].concat(_toConsumableArray(_affected)),
        resultAffected: []
      };
    }
    return undefined;
  });

  cycleMap = new WeakMap();
  userShouldDive = options.deproxifyShouldDive || returnTrue;
  var result = deproxifyResult(proxyMap, preResult, affected, true);
  var cacheLine = { args: args, affected: affected, result: result, callArgs: callArgs };
  if (cache.length < memoizationDepth) {
    cache.push(cacheLine);
  } else {
    (0, _cache.updateCacheLine)(cache, 0, cacheLine);
  }

  if (spreadDetected.length > 0) {
    if (process.env.NODE_ENV !== 'production') {
      console.warn('memoize-state: object spread detected in ', func, '. Keys affected: ', spreadDetected.map(function (key) {
        return key ? key : 'root';
      }), '. This is no-op.');
    }
  }

  return result;
}